#include <fstream>
#include <iostream>
#include <qstringlist.h>
#include <qdir.h>
#include <qprocess.h>
#include "appctl.h"

using namespace std;

#define DEFAULT_CONFIG_D_DIR "/etc/pcontrol/pcontrol.d"
#define DEFAULT_CONFIG "/etc/pcontrol/appctl.conf"

#define TOKEN_ENABLE "enable="


static int readAppConfig(const char *config, bool *enabled)
{
	int ret;
	char *p;
	char line[100];

	ret = -1;
	ifstream infile(config);
	if (!infile)
		goto out;

	infile.seekg (0, ios::beg);
	while(infile.getline(line, 100)) {
		if ((p = strcasestr(line, TOKEN_ENABLE)) != NULL) {
			char *ptr;
			ptr = &p[strlen(TOKEN_ENABLE)];
			if ((p = strcasestr(ptr, "yes")) != NULL)
				*enabled = true;
			else 
				*enabled = false;
			break;
		}
	}

	ret = 0;
out:
	infile.close();
	return ret;
}


static int writeAppConfig(const char *config, bool enabled)
{
	const char tmpfile[] = "/tmp/.pcontrol.conf.tmp";
	char line[100];
	char *buffer;
	char *p;
	int size;

	if (!config)
		return -1;

	ifstream infile(config);
	/* FIXME: use mkstemp */
	ofstream outfile(tmpfile);

	if (!infile || !outfile)
		return -2;

	size = sizeof(line) + 1;
	buffer = new char[size];
	if (!buffer)
		return -3;

	while(infile.getline(line, 100)) {
		if ((p = strcasestr(line, TOKEN_ENABLE)) != 0) {
			/* overwrite the config line */
			snprintf(buffer, size, "enable=\"%s\"\n", 
					enabled ? "yes" : "no"); 
			outfile.write(buffer, strlen(buffer));
		}
		else {
			snprintf(buffer, size, "%s\n", line);
			outfile.write(buffer, strlen(buffer));
		}
	}

	outfile.close();
	infile.close();
	delete buffer;

	infile.open(tmpfile);
	outfile.open(config, ios::trunc);

	infile.seekg(0, ifstream::end);
	size = infile.tellg();
	infile.seekg(0);

	// allocate memory for file content
	buffer = new char [size];

	// read content of infile
	infile.read(buffer, size);

	// write to outfile
	outfile.write(buffer, size);
  
	// release dynamically-allocated memory
	delete buffer;

	outfile.close();
	infile.close();

	unlink(tmpfile);

	return 0;
}


AppControl::AppControl()
{
	this->enabled = false;
	this->applist = new QPtrList<Application>();
	this->open();
}


AppControl::~AppControl()
{
	this->close();
	delete this->applist;
}


int AppControl::open()
{
	int ret;
	bool i;
	const char *filename;
	char config[255];

	i = false;
	ret = readAppConfig(DEFAULT_CONFIG, &i);
	if (ret != 0)
		return ret;

	this->enabled = i;

	QDir dir(QString(DEFAULT_CONFIG_D_DIR));
	dir.setFilter(QDir::Files | QDir::Readable);	
	dir.setSorting(QDir::Name);

	QStringList list = dir.entryList();
	for (QStringList::Iterator it = list.begin(); it != list.end(); ++it ) {
		filename = *it;
		snprintf(config, 255, "%s/%s", DEFAULT_CONFIG_D_DIR, filename);
		applist->append(new Application(config));
	}

	return ret;
}


int AppControl::close()
{

	restartService();
	applist->clear();

	return 0;
}


void AppControl::setEnabled()
{
	this->enabled = true;
}


void AppControl::setDisabled()
{
	this->enabled = false;
}

bool AppControl::isEnabled()
{
	return this->enabled;
}


QPtrList<Application> *AppControl::getAppList()
{
	return this->applist;
}


bool AppControl::restartService()
{
	// sync config file
	writeAppConfig(DEFAULT_CONFIG, this->enabled);

	// sync services files
	Application *app;
	for (app = applist->first(); app; app = applist->next())
		app->write();

	// restart controller
	QProcess ctldaemon(QString("/usr/sbin/pcontrol"));
	ctldaemon.addArgument(QString("restart"));
	if (!ctldaemon.start()) {
		qWarning("Can't restart application controller");
		return false;
	}

	return true;
}


