
#include <fstream>
#include <iostream>
#include <qmessagebox.h>

#include "application.h"


#define TOKEN_NAME	"NAME="
#define TOKEN_SUNDAY	"SUNDAY="
#define TOKEN_MONDAY	"MONDAY="
#define TOKEN_TUESDAY	"TUESDAY="
#define TOKEN_WEDNESDAY	"WEDNESDAY="
#define TOKEN_THURSDAY	"THURSDAY="
#define TOKEN_FRIDAY	"FRIDAY="
#define TOKEN_SATURDAY	"SATURDAY="

int extract_name(const char *line, char **name)
{
	/* line = "name" */
	char *s, *pb, *pe;

	s = strdup(line);
	pb = s;
	// skip first "
	pb++;
	// find last "
	pe = strstr(pb, "\""); 
	*pe = '\0';

	*name = strdup(pb);
	delete(s);
	
	return 0;
}

int extract_rule(const char *line, QPtrList<Rule> *list)
{
	/* "14:43-16:00 17:00-18:00" */
	char *s, *pb, *pe;

	if (!line) {
		qWarning("No string to extract rule");
		return -1;
	}

	s = strdup(line);
	pb = s;

	if (*pb == '\0') {
		delete s;
		return 0;
	}

	// skip first "
	pb++;

	// skip empty rules
	if (pb[0] == '"') {
		delete s;
		return 0;
	}

	// find space
	while ((pe = strstr(pb, " ")) != NULL) {
		*pe = '\0';
		list->append(new Rule(pb));
		pb = ++pe;
	}

	// find last " 
	pe = strstr(pb, "\""); 
	*pe = '\0';

	list->append(new Rule(pb));

	delete s;
	return 0;
}


char *assemble_rule(QPtrList<Rule> *list)
{
	const char *rstr;
	char *result;
	char *resptr;
	Rule *r;
	int sz;

	sz = list->count() * (23 + 1) + 1;  /* "14:43-16:00 17:00-18:00" */

	result = new char[sz];
	if (!result)
		return NULL;

	memset(result, 0, sz);
	resptr = result;
	for (r = list->first(); r; r = list->next()) {
		rstr = r->getString();
		sprintf(resptr, "%s ", rstr);
		resptr += strlen(rstr) + 1;
	}

	// remove last space
	--resptr;
        *resptr = '\0';
	return result;
}

Application::Application(const char *config)
{
	name = NULL;
	openned = false;
	this->config = strdup(config);
	for (int i = MONDAY; i <= SUNDAY; i++)
		weekList[i] = new QPtrList<Rule>();

	this->read();
}


Application::~Application()
{
	this->write();
}


int Application::read()
{
	int ret;
	char *p;
	char line[512];

	if (!config)
		return -1;

	ret = -1;
	std::ifstream infile(config);
	if (!infile)
		goto out;

	infile.seekg (0, std::ios::beg);
	while(infile.getline(line, 512)) {
		if ((p = strcasestr(line, TOKEN_NAME)) != NULL) {
			extract_name(&p[strlen(TOKEN_NAME)], &name);
			continue;
		}

		if ((p = strcasestr(line, TOKEN_MONDAY)) != NULL) {
			extract_rule(&p[strlen(TOKEN_MONDAY)], 
					weekList[MONDAY]);
			continue;
		}

		if ((p = strcasestr(line, TOKEN_TUESDAY)) != NULL) {
			extract_rule(&p[strlen(TOKEN_TUESDAY)], 
					weekList[TUESDAY]);
			continue;
		}
		if ((p = strcasestr(line, TOKEN_WEDNESDAY)) != NULL) {
			extract_rule(&p[strlen(TOKEN_WEDNESDAY)], 
					weekList[WEDNESDAY]);
			continue;
		}
		if ((p = strcasestr(line, TOKEN_THURSDAY)) != NULL) {
			extract_rule(&p[strlen(TOKEN_THURSDAY)], 
					weekList[THURSDAY]);
			continue;
		}
		if ((p = strcasestr(line, TOKEN_FRIDAY)) != NULL) {
			extract_rule(&p[strlen(TOKEN_FRIDAY)], 
					weekList[FRIDAY]);
			continue;
		}
		if ((p = strcasestr(line, TOKEN_SATURDAY)) != NULL) {
			extract_rule(&p[strlen(TOKEN_SATURDAY)], 
					weekList[SATURDAY]);
			continue;
		}
		if ((p = strcasestr(line, TOKEN_SUNDAY)) != NULL) {
			extract_rule(&p[strlen(TOKEN_SUNDAY)], 
					weekList[SUNDAY]);
			continue;
		}

	}

	ret = 0;
out:
	infile.close();
	return ret;
	
	openned = true;
	return 0;
}

bool Application::write()
{
	const char tmpfile[] = "/tmp/.sites.conf";
	char line[512];
	char *buffer;
	char *p;
	int size;

	if (!config)
		return false;

	std::ifstream infile(config);
	/* FIXME: use mkstemp */
	std::ofstream outfile(tmpfile);

	if (!infile || !outfile)
		return false;

	size = sizeof(line) + 1;
	buffer = new char[size];
	if (!buffer)
		return false;

	while(infile.getline(line, 512)) {
		if ((p = strcasestr(line, TOKEN_MONDAY)) != NULL) {
			char *rline = assemble_rule(weekList[MONDAY]);
			snprintf(buffer, size, "%s\"%s\"\n", TOKEN_MONDAY, 
					rline); 
			outfile.write(buffer, strlen(buffer));
			delete rline;
			continue;
		}

		if ((p = strcasestr(line, TOKEN_TUESDAY)) != NULL) {
			char *rline = assemble_rule(weekList[TUESDAY]);
			snprintf(buffer, size, "%s\"%s\"\n", TOKEN_TUESDAY, 
					rline); 
			outfile.write(buffer, strlen(buffer));
			delete rline;
			continue;
		}
		if ((p = strcasestr(line, TOKEN_WEDNESDAY)) != NULL) {
			char *rline = assemble_rule(weekList[WEDNESDAY]);
			snprintf(buffer, size, "%s\"%s\"\n", TOKEN_WEDNESDAY, 
					rline); 
			outfile.write(buffer, strlen(buffer));
			delete rline;
			continue;
		}
		if ((p = strcasestr(line, TOKEN_THURSDAY)) != NULL) {
			char *rline = assemble_rule(weekList[THURSDAY]);
			snprintf(buffer, size, "%s\"%s\"\n", TOKEN_THURSDAY, 
					rline); 
			outfile.write(buffer, strlen(buffer));
			delete rline;
			continue;
		}

		if ((p = strcasestr(line, TOKEN_FRIDAY)) != NULL) {
			char *rline = assemble_rule(weekList[FRIDAY]);
			snprintf(buffer, size, "%s\"%s\"\n", TOKEN_FRIDAY, 
					rline); 
			outfile.write(buffer, strlen(buffer));
			delete rline;
			continue;
		}
		if ((p = strcasestr(line, TOKEN_SATURDAY)) != NULL) {
			char *rline = assemble_rule(weekList[SATURDAY]);
			snprintf(buffer, size, "%s\"%s\"\n", TOKEN_SATURDAY, 
					rline); 
			outfile.write(buffer, strlen(buffer));
			delete rline;
			continue;
		}
		if ((p = strcasestr(line, TOKEN_SUNDAY)) != NULL) {
			char *rline = assemble_rule(weekList[SUNDAY]);
			snprintf(buffer, size, "%s\"%s\"\n", TOKEN_SUNDAY,
					rline); 
			outfile.write(buffer, strlen(buffer));
			delete rline;
			continue;
		}

		snprintf(buffer, size, "%s\n", line);
		outfile.write(buffer, strlen(buffer));
	}

	outfile.close();
	infile.close();
	delete buffer;

	infile.open(tmpfile);
	outfile.open(config, std::ios::trunc);

	infile.seekg(0, std::ifstream::end);
	size = infile.tellg();
	infile.seekg(0);

	// allocate memory for file content
	buffer = new char [size];

	// read content of infile
	infile.read(buffer, size);

	// write to outfile
	outfile.write(buffer, size);
  
	// release dynamically-allocated memory
	delete buffer;

	outfile.close();
	infile.close();

	unlink(tmpfile);

	return true;
}


const char *Application::getConfig()
{
	return this->config;
}


const char *Application::getName()
{
	return this->name;
}


QPtrList<Rule> *Application::getRules(int weekday)
{
	if (weekday < MONDAY || weekday > SUNDAY)
		return NULL;

	return this->weekList[weekday];
}


bool Application::addRule(int weekday, const char *stimestr,
				const char *ftimestr)
{
	QPtrList<Rule> *list;
	class Rule *rule;
	class Rule *radd;
	RTime *raddST, *raddFT;
	RTime *ruleST, *ruleFT;
	QString terror;

	radd = new Rule(stimestr, ftimestr);

	list = this->weekList[weekday];
	raddST = radd->getStartTime();
	raddFT = radd->getFinishTime();
	for (rule = list->first(); rule; rule = list->next()) {
		if (*radd == *rule) {
			terror = "Error: Duplicated rule";
			goto out_error;
		}

		ruleST = rule->getStartTime();
		ruleFT = rule->getFinishTime();
		// check if end have the same begin as other rule
		if (*raddFT == *ruleST) {
			terror = "Colision: new rule finish time conflicts with another rule";
			goto out_error;
		}

		if (*raddST == *ruleFT) {
			terror = "Colision: new rule start time conflicts with another rule";
			goto out_error;
		}

		if (*raddFT > *ruleST && *raddFT < *ruleFT) {
			terror = "Colision: new rule finish time overlap another rule";
			goto out_error;
		}

		if (*raddST > *ruleST && *raddFT < *ruleFT) {
			terror = "Colision: new rule is inside of another rule";
			goto out_error;
		}

		if (*raddST > *ruleST && *raddST < *ruleFT) {
			terror = "Colision: new rule start time overlap another rule";
			goto out_error;
		}
	}

	list->append(radd);
	return true;

out_error:
	delete radd;
	QMessageBox::critical( 0, "Rule Time Error", terror);
	return false;
}


bool Application::delRule(int weekday, const char *stimestr, 
				const char *ftimestr)
{
	QPtrList<Rule> *list;
	class Rule *rule;
	Rule rdel(stimestr, ftimestr);
	
	list = this->weekList[weekday];
	for (rule = list->first(); rule; rule = list->next()) {
		if (rdel == *rule) {
			list->remove(rule);
			return true;
		}
	}

	qWarning("Cant find rule object, list corrupted?");
	return false;
}


