
#include <iostream>
#include <fstream>
#include <qstringlist.h>
#include "bwords.h"


#define CONFIG_FILE "/etc/dansguardian/lists/bannedphraselist"

BWords::BWords()
 : QStringList()
{
	config = CONFIG_FILE;
	sort();
}


BWords::~BWords()
{
	clear();
}


QStringList *BWords::getList()
{
	return this;
}


bool BWords::writeConfig()
{
	const char tmpfile[] = "/tmp/.sites.conf";
	char line[100];
	char *buffer;
	int size;

	if (!config)
		return false;

	std::ifstream infile(config);
	/* FIXME: use mkstemp */
	std::ofstream outfile(tmpfile);

	if (!infile || !outfile)
		return false;

	size = sizeof(line) + 1;
	buffer = new char[size];
	if (!buffer)
		return false;

	while(infile.getline(line, 100)) {
		// write comment
		if (line[0] == '#') {
			snprintf(buffer, size, "%s\n", line);
			outfile.write(buffer, strlen(buffer));
			continue;
		}

		// write default .include rules
		if (line[0] == '.') {
			snprintf(buffer, size, "%s\n", line);
			outfile.write(buffer, strlen(buffer));
			continue;
		}

		QString l = line;
		QString site = l.stripWhiteSpace(); 
		// write blank lines
		if (site.isEmpty()) {
			snprintf(buffer, size, "\n");
			outfile.write(buffer, strlen(buffer));
		}

	}


	for (QStringList::Iterator it = begin(); it != end(); ++it) {
		QString site = *it;
		snprintf(buffer, size, "<%s>\n", site.ascii()); 
		outfile.write(buffer, strlen(buffer));
	}

	outfile.close();
	infile.close();
	delete buffer;

	infile.open(tmpfile);
	outfile.open(config, std::ios::trunc);

	infile.seekg(0, std::ifstream::end);
	size = infile.tellg();
	infile.seekg(0);

	// allocate memory for file content
	buffer = new char [size];

	// read content of infile
	infile.read(buffer, size);

	// write to outfile
	outfile.write(buffer, size);
  
	// release dynamically-allocated memory
	delete buffer;

	outfile.close();
	infile.close();

	unlink(tmpfile);

	clear();

	return true;
}


bool BWords::readConfig()
{
	char line[100];

	if (!config)
		return false;

	std::ifstream infile(config);
	if (!infile)
		return false;

	infile.seekg (0, std::ios::beg);
	while(infile.getline(line, 100)) {
		// skip comment
		if (line[0] == '#')
			continue;

		// skip defaults .include
		if (line[0] == '.')
			continue;

		QString site = line;
		site = site.stripWhiteSpace(); 
		// skip blank lines
		if (site.isEmpty())
			continue;

		site = site.remove('>');
		site = site.remove('<');
		
		// insert on list
		append(site);
	}

	infile.close();
	return true;
}


