
#include <iostream>
#include <fstream>
#include "filter.h"

#include <qprocess.h>

using namespace std;

#define FW_HELPER "/usr/sbin/pcontrol-fw.sh"
#define DEFAULT_CONFIG "/etc/pcontrol/filter.conf"
#define TOKEN_ENABLE "enable="

static int readFilterConfig(const char *config, bool *enabled)
{
	int ret;
	char *p;
	char line[100];

	ret = -1;
	ifstream infile(config);
	if (!infile)
		goto out;

	infile.seekg (0, ios::beg);
	while(infile.getline(line, 100)) {
		if ((p = strcasestr(line, TOKEN_ENABLE)) != NULL) {
			char *ptr;
			ptr = &p[strlen(TOKEN_ENABLE)];
			if ((p = strcasestr(ptr, "yes")) != NULL)
				*enabled = true;
			else 
				*enabled = false;
			break;
		}
	}

	ret = 0;
out:
	infile.close();
	return ret;
}


static int writeFilterConfig(const char *config, bool enabled)
{
	const char tmpfile[] = "/tmp/.filter.conf.tmp";
	char line[100];
	char *buffer;
	char *p;
	int size;

	if (!config)
		return -1;

	ifstream infile(config);
	/* FIXME: use mkstemp */
	ofstream outfile(tmpfile);

	if (!infile || !outfile)
		return -2;

	size = sizeof(line) + 1;
	buffer = new char[size];
	if (!buffer)
		return -3;

	while(infile.getline(line, 100)) {
		if ((p = strcasestr(line, TOKEN_ENABLE)) != 0) {
			/* overwrite the config line */
			snprintf(buffer, size, "enable=\"%s\"\n", 
					enabled ? "yes" : "no"); 
			outfile.write(buffer, strlen(buffer));
		}
		else {
			snprintf(buffer, size, "%s\n", line);
			outfile.write(buffer, strlen(buffer));
		}
	}

	outfile.close();
	infile.close();
	delete buffer;

	infile.open(tmpfile);
	outfile.open(config, ios::trunc);

	infile.seekg(0, ifstream::end);
	size = infile.tellg();
	infile.seekg(0);

	// allocate memory for file content
	buffer = new char [size];

	// read content of infile
	infile.read(buffer, size);

	// write to outfile
	outfile.write(buffer, size);
  
	// release dynamically-allocated memory
	delete buffer;

	outfile.close();
	infile.close();

	unlink(tmpfile);

	return 0;
}


Filter::Filter()
{
	this->filterEnabled = false;
}


Filter::~Filter()
{
}


void Filter::setFilterEnabled()
{
	this->filterEnabled = true;
}
	

bool Filter::isFilterEnabled()
{
	return this->filterEnabled;
}


void Filter::setFilterDisabled()
{
	this->filterEnabled = false;
}


int Filter::readConfig()
{
	int ret;
	bool i;

	ret = readFilterConfig(DEFAULT_CONFIG, &i);
	if (ret != 0)
		return ret;

	this->filterEnabled = i;
	return ret;
}




int Filter::writeConfig()
{
	writeFilterConfig(DEFAULT_CONFIG, this->filterEnabled);
	return 0;
}


int Filter::restartService()
{
        QProcess daemon(QString("/etc/init.d/dansguardian"));
	daemon.addArgument(QString("restart"));
	if (!daemon.start()) {
		qWarning("Can't restart dansguardian");
		return -1;
	}

	if (filterEnabled) {
        	QProcess daemon(QString(FW_HELPER));
		daemon.addArgument(QString("filter"));
		if (!daemon.start()) {
			qWarning("Can't setup filter firewall");
			return -1;
		}
	}
	else {
        	QProcess daemon(QString(FW_HELPER));
		daemon.addArgument(QString("proxy"));
		if (!daemon.start()) {
			qWarning("Can't setup proxy firewall");
			return -1;
		}
	}

	return 0;
}


