

#include <qdatetimeedit.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qcombobox.h>


#include "myaddedit.h"
#include "weekdays.h"
#include "mywin.h"
#include "appctl.h"
#include "rtime.h"

myaddedit::myaddedit(mywin *parent, const QString *name)
 : addedit(parent, 0, true)
{
	rlappname->setText(*name);
	for (int week = MONDAY; week <= SUNDAY; week++)
		cbweekday->insertItem(QDate::longDayName(week));

	cbdpolicy->insertItem(QString("Accept"));
	cbdpolicy->insertItem(QString("Deny"));
	cbdpolicy->setDisabled(true);
}


myaddedit::myaddedit(mywin *parent, const QString *name,
		    const QString *weekday,
		    class Rule *rule)
 : addedit(parent, 0, true)
{
	const RTime *tm;

	rlappname->setText(*name);
	for (int week = MONDAY; week <= SUNDAY; week++)
		cbweekday->insertItem(QDate::longDayName(week));

	cbweekday->setCurrentText(*weekday);
	cbdpolicy->insertItem(QString("Accept"));
	cbdpolicy->insertItem(QString("Deny"));
	cbdpolicy->setDisabled(true);

	
	tm = rule->getStartTime();
	stime->setTime(*tm);
	tm = rule->getFinishTime();
	ftime->setTime(*tm);
}


myaddedit::~myaddedit()
{
}


/* 10 minutes interval */
QTime myaddedit::fixTime(QTime t)
{
	int hour = t.hour();
	int minute = t.minute();
	int r = minute % 10;

	if (r > 5) 
		minute = ((minute/10) + 1) * 10 ;
	else
		minute = (minute/10) * 10 ;

	if (minute > 59) {
		hour += 1;
		minute = 0;
	}

	if (hour == 24) {
		hour = 23;
		minute = 59;
	}

	return QTime(hour, minute);
}

RTime myaddedit::getStime()
{
	return RTime(fixTime(stime->time()));
}


RTime myaddedit::getFtime()
{
	return RTime(fixTime(ftime->time()));
}


const QString myaddedit::getWeekdayName()
{
	return cbweekday->currentText();
}


#include "myaddedit.moc"
