


#include <ksystemtray.h>
#include <kpopupmenu.h>
#include <klocale.h>

#include <qvariant.h>
#include <qpushbutton.h>
#include <qtabwidget.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qspinbox.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qlistbox.h>
#include <qheader.h>
#include <qlistview.h>
#include <qcombobox.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qmessagebox.h>
#include <qprocess.h>

#include "rtime.h"
#include "myaddedit.h"
#include "tiny.h"
#include "mywin.h"
#include "filter.h"
#include "bsites.h"
#include "bwords.h"
#include "inputurl.h"
#include "inputword.h"
#include "passwin.h"

#define FW_HELPER "/usr/sbin/pcontrol-fw.sh"

mywin::mywin(QWidget *parent, const char *name)
 : mwin(parent, name)
{
	admMode = false;
	pbAdmMode->setText(QString("Admin &Mode"));
	leProxy->setDisabled(true);
	sbPortNumber->setDisabled(true);
	cbAppSel->setDisabled(true);
	cbAppEnable->setDisabled(true);
	pbAppDel->setDisabled(true);
	pbAppAdd->setDisabled(true);
	pbAppEdit->setDisabled(true);
	lvAppSched->setDisabled(true);
	cbSite->setDisabled(true);
	pbDelSite->setDisabled(true);
	pbInsSite->setDisabled(true);
	cbWord->setDisabled(true);
	pbDelWord->setDisabled(true);
	pbInsWord->setDisabled(true);
	cbFilterEnable->setDisabled(true);

	// system tray
	systray = new KSystemTray(this, i18n("Parental Control"));
	systray->setCaption(i18n("Parental Control"));
	systray->setPixmap( systray->loadIcon("userconfig"));
	systmenu = systray->contextMenu();
	QObject::connect(systray, SIGNAL( quitSelected() ), 
			this, SLOT( close() ));

	QObject::connect(systmenu, SIGNAL( activated (int) ), 
			this, SLOT( SysMenu(int) ));
	
	// proxy
	tiny = new TinyProxy();
	tiny->readConfig();
	if (tiny->isProxyEnabled()) {
		spopup_proxy = systmenu->insertItem(QString(i18n("Disable Proxy")));
		cbProxyEnable->setChecked(true);
	}
	else {
		spopup_proxy = systmenu->insertItem(QString(i18n("Enable Proxy")));
		cbProxyEnable->setChecked(false);
	}

	connect( cbProxyEnable, SIGNAL( stateChanged(int) ), 
			this, SLOT( proxyEnable (int) ) );

	// content filtering 
	filter = new Filter();
	filter->readConfig();
	if (filter->isFilterEnabled())
		cbFilterEnable->setChecked(true);
	else
		cbFilterEnable->setChecked(false);

	connect( cbFilterEnable, SIGNAL( stateChanged(int) ), 
			this, SLOT( filterEnable (int) ) );

	leProxy->setText(QString(tiny->getProxy()));
	sbPortNumber->setValue(tiny->getPort());
	
	// application controller
	appctl = new AppControl();
	if (appctl->isEnabled())
		cbAppEnable->setChecked(true);
	else
		cbAppEnable->setChecked(false);

	connect( cbAppEnable, SIGNAL( stateChanged(int) ), 
			this, SLOT( appEnable(int) ) );

	loadAppList();
	loadAppRules();

	// Sites Black Listed 
	bsites = new BSites();

	// Words Black Listed 
	bwords = new BWords();

	systray->show();
}


mywin::~mywin()
{
}


void mywin::proxyEnable(int )
{ 
	if (tiny->isProxyEnabled()) {
		tiny->setProxyDisabled();
		cbProxyEnable->setChecked(false);
		QProcess daemon(QString(FW_HELPER));
		daemon.addArgument(QString("stop"));
		if (!daemon.start()) {
			qWarning("Can't stop and clean the proxy settings");
			return;
		}

		systmenu->changeItem(spopup_proxy, 
				QString(i18n("Enable Proxy")));
 	} 
	else {
		tiny->setProxyEnabled();
		cbProxyEnable->setChecked(true);
		systmenu->changeItem(spopup_proxy, 
				QString(i18n("Disable Proxy")));

	}
}


void mywin::SysMenu(int id)
{
	if (spopup_proxy == id) {
		cbProxyEnable->disconnect(SIGNAL( stateChanged(int)), this, 
				SLOT( proxyEnable(int) ) );
		proxyEnable(0);
		connect( cbProxyEnable, SIGNAL( stateChanged(int) ), this, 
				SLOT( proxyEnable(int) ) );
	}
}

void mywin::appProxy(const QString &qsproxy)
{
	QString proxy = qsproxy.lower();
	const char *px = proxy;
	char *ps, *pb, *pe;

	ps = strdup(px);

	if ((pb = strstr(ps, "http://")) != NULL)
		pb += strlen("http://");
	else
		pb = ps;

	pe = pb;

	if ((pe = strstr(pb, ":")) != NULL)
		*pe = '\0';
	
	tiny->setProxy(pb);

	delete ps;
}


void mywin::appProxyPort(int port)
{
	tiny->setPort(port);
}


void mywin::siteChanged(int, int) { return ; }


void mywin::deleteSite()
{ 
	int id = cbSite->currentItem();
	QString site = cbSite->text(id);
	// remove from object list 
	bsites->remove(site);
	// remove from list view 
	cbSite->removeItem(id);
}


void mywin::insertSite()
{ 

	InputURL url;
	
	url.qlinput->setText(QString("URL:"));
	if (url.exec() != QDialog::Accepted)
		return;

	QString newurl = url.leData->text().stripWhiteSpace();

	// FIXME: parse and check the input
	bsites->append(newurl);
	cbSite->insertItem(newurl);
}


void mywin::wordChanged(int, int)  { return ; }

void mywin::insertWord()
{
	InputWord word;
	
	word.qlinput->setText(QString("Word:"));
	if (word.exec() != QDialog::Accepted)
		return;

	QString newword = word.leData->text().stripWhiteSpace();

	// FIXME: parse and check the input
	bwords->append(newword);
	cbWord->insertItem(newword);
}


void mywin::deleteWord()
{
	int id = cbWord->currentItem();
	QString word = cbWord->text(id);
	// remove from object list 
	bwords->remove(word);
	// remove from list view 
	cbWord->removeItem(id);
	return;
}


void mywin::adminMode()
{
	if (admMode) {
		admMode = false;
		pbAdmMode->setText(QString("Admin &Mode"));
		leProxy->setDisabled(true);
		sbPortNumber->setDisabled(true);
		cbAppSel->setDisabled(true);
		cbAppEnable->setDisabled(true);
		pbAppDel->setDisabled(true);
		pbAppAdd->setDisabled(true);
		pbAppEdit->setDisabled(true);
		lvAppSched->setDisabled(true);
		cbSite->setDisabled(true);
		cbWord->setDisabled(true);
		pbDelSite->setDisabled(true);
		pbInsSite->setDisabled(true);
		pbDelWord->setDisabled(true);
		pbInsWord->setDisabled(true);
		cbFilterEnable->setDisabled(true);
		// sync words black list
		bwords->writeConfig();
		cbWord->clear();
		// sync sites black list
		bsites->writeConfig();
		cbSite->clear();
		// sync filter configuration
		filter->writeConfig();
		filter->restartService();
		// sync proxy configuration
		tiny->writeConfig();
		tiny->restartService();
		// app controller sync
		appctl->restartService();
	}
	else {
		QStringList userList;
		userList.append("root");
		userList.append("parental");
		PassWin pwin(this, "passwin", userList);  
		if ( pwin.exec() != QDialog::Accepted )
			return;

		if (pwin.valid_auth() == false) {
			QMessageBox::critical( 0, "Error", 
					QString("Password Mismatch"));
			return;
		}

		admMode = true;
		pbAdmMode->setText(QString("&Apply"));
		leProxy->setDisabled(false);
		sbPortNumber->setDisabled(false);
		cbAppSel->setDisabled(false);
		cbAppEnable->setDisabled(false);
		pbAppDel->setDisabled(false);
		pbAppAdd->setDisabled(false);
		pbAppEdit->setDisabled(false);
		lvAppSched->setDisabled(false);
		cbSite->setDisabled(false);
		cbWord->setDisabled(false);
		pbDelSite->setDisabled(false);
		pbInsSite->setDisabled(false);
		pbDelWord->setDisabled(false);
		pbInsWord->setDisabled(false);
		cbFilterEnable->setDisabled(false);
		bwords->readConfig();
		loadBWords();
		bsites->readConfig();
		loadBSites();
	}
}

void mywin::appEnable(int state)
{
	if (QButton::Off == state) {
		appctl->setDisabled();
		return;
 	} 
	 
	if (QButton::On == state) {
		appctl->setEnabled();
		return;
	}
}


void mywin::loadAppList()
{
	QPtrList<Application> *list;
	Application *app;

	list = appctl->getAppList();
	for (app = list->first(); app; app = list->next())
		cbAppSel->insertItem(QString(app->getName()));
}


int __loadAppRules(class Application *app, QListViewItem *parent, int weekday)
{
	QPtrList<Rule> *list;
	class Rule *rule;
	
	if (!parent)
		return -1;
	
	list = app->getRules(weekday);
	for (rule = list->first(); rule; rule = list->next()) {
		RTime *startTime = rule->getStartTime(); 
		RTime *finishTime = rule->getFinishTime(); 
		parent->insertItem(new QListViewItem( parent, 
					QString(""),
					QString("Deny"),
					startTime->toText(),
					finishTime->toText()));
	}

	parent->setOpen(true);

	return 0;
}


void mywin::_loadAppRules(class Application *app)
{
	int weekday;
	QListViewItem *day;


	lvAppSched->clear();
	lvAppSched->setSorting(-1);
	lvAppSched->setRootIsDecorated(true);
	
	for (weekday = SUNDAY; weekday >= MONDAY; weekday--) {
		day = new QListViewItem( lvAppSched, 
					QDate::longDayName(weekday), 
					QString("Accept"));

		__loadAppRules(app, day, weekday);
	}

}


void mywin::loadAppRules()
{
	QPtrList<Application> *list;
	Application *app;
	QString appname;

	appname = cbAppSel->currentText();

	list = appctl->getAppList();
	for (app = list->first(); app; app = list->next())
		if (QString(app->getName()) == appname) {
			_loadAppRules(app);
			return;
		}

	qWarning(" No application object found. List corrupted?");
}



void mywin::appSelection(const QString& )
{
	loadAppRules();
}


void mywin::appDelRule()
{
	QListViewItem *parentItem;
	QListViewItem *ruleItem;
	QString weekdaystr;
	int weekday;
	QString appname;
	QPtrList<Application> *alist;
	Application *app;

	ruleItem = lvAppSched->currentItem();
	
	// FIXME: show an error message
	if (!ruleItem->text(0).isEmpty()) {
		qWarning("Cant delete weekday default policy");
		return;
	}

	// find out what weekday 
	parentItem = ruleItem->parent();
	if (parentItem == NULL) {
		qWarning("Parentless rule");
		return;
	}

	weekdaystr = parentItem->text(0);
	for (weekday = MONDAY; weekday <= SUNDAY; weekday++)
		if (QDate::longDayName(weekday) == weekdaystr)
			break;

	// find out what application
	appname = cbAppSel->currentText();

	// find out what application object and delete the rule
	alist = appctl->getAppList();
	for (app = alist->first(); app; app = alist->next())
		if (QString(app->getName()) == appname)
			break;

	if (!app) {
		qWarning("Cant find application object, list corrupted?");
		return;
	}

	app->delRule(weekday, ruleItem->text(2), ruleItem->text(3)); 
	loadAppRules();
}


bool mywin::checkTime(const RTime start, const RTime finish)
{
	if (start == finish) {
		QMessageBox::critical( 0, "Time Error", 
			QString("Error: start and finish time are equal"));
		return false;
	}

	if (start > finish) {
		QMessageBox::critical( 0, "Time Error", 
			QString("Error: start time after finish time"));
		return false;
	}

	return true;
}


void mywin::appAddRule()
{
	QString appname;
	QPtrList<Application> *alist;
	Application *app;
	int wday;

	appname = cbAppSel->currentText();
	myaddedit addwin(this, &appname);
	if ( addwin.exec() != QDialog::Accepted )
		return;

	const QString weekday = addwin.getWeekdayName();
	RTime stime = addwin.getStime();
	RTime ftime = addwin.getFtime();

	if (!checkTime(stime, ftime))
		return;

	for (wday = MONDAY; wday <= SUNDAY; wday++)
		if (QDate::longDayName(wday) == weekday)
			break;

	// find out what application object and add the rule
	alist = appctl->getAppList();
	for (app = alist->first(); app; app = alist->next())
		if (QString(app->getName()) == appname)
			break;

	if (!app) {
		qWarning("Cant find application object, list corrupted?");
		return;
	}

	app->addRule(wday, stime.toText(), ftime.toText()); 
	loadAppRules();
}


void mywin::appEditRule()
{
	QListViewItem *parentItem;
	QListViewItem *ruleItem;
	QString appname;
	QPtrList<Application> *alist;
	Application *app;
	QString weekday;
	class Rule *redit;
	int wday;

	appname = cbAppSel->currentText();

	ruleItem = lvAppSched->currentItem();
	// FIXME: show an error message
	if (!ruleItem->text(0).isEmpty()) {
		qWarning("Cant edit weekday default policy");
		return;
	}

	// find out what weekday 
	parentItem = ruleItem->parent();
	if (parentItem == NULL) {
		qWarning("Error: Parentless rule");
		return;
	}

	weekday = parentItem->text(0);

	// find out what application
	appname = cbAppSel->currentText();

	// find out what application object and delete the rule
	alist = appctl->getAppList();
	for (app = alist->first(); app; app = alist->next())
		if (QString(app->getName()) == appname)
			break;

	if (!app) {
		qWarning("Error: Cant find application object, list corrupted?");
		return;
	}

	redit = new Rule(ruleItem->text(2), ruleItem->text(3));
	myaddedit addwin(this, &appname, &weekday, redit);
	if ( addwin.exec() != QDialog::Accepted )
		return;

	const QString newweekday = addwin.getWeekdayName();
	RTime stime = addwin.getStime();
	RTime ftime = addwin.getFtime();

	if (!checkTime(stime, ftime))
		return;

	for (wday = MONDAY; wday <= SUNDAY; wday++)
		if (QDate::longDayName(wday) == weekday)
			break;

	app->delRule(wday, ruleItem->text(2), 
				ruleItem->text(3)); 

	for (wday = MONDAY; wday <= SUNDAY; wday++)
		if (QDate::longDayName(wday) == newweekday)
			break;

	app->addRule(wday, stime.toText(), ftime.toText()); 
	loadAppRules();
}


void mywin::appExit()
{ 
	/*delete appctl;
	delete tiny;
	delete filter;*/
	if (admMode) {
		bwords->writeConfig();
		bsites->writeConfig();
	}
/*
	delete bsites;
	delete bwords;
	this->close();
	*/
	hide();
}


void mywin::loadBSites()
{
	cbSite->clear();
	for (QStringList::Iterator it = bsites->begin(); it != bsites->end(); ++it)
		cbSite->insertItem(*it); 
}


void mywin::loadBWords()
{
	QStringList *list = bwords->getList();
	cbWord->clear();
	for (QStringList::Iterator it = list->begin(); it != list->end(); ++it)
		cbWord->insertItem(*it); 
}


void mywin::filterEnable(int )
{
	if (filter->isFilterEnabled()) {
		filter->setFilterDisabled();
 	}
	else {	
		filter->setFilterEnabled();
	}
}

void mywin::fixSize(QShowEvent *e)
{
	resize(800,480);
	mwin::showEvent(e);
}

#include "mywin.moc"
