
#include <security/pam_appl.h>
#include <security/pam_misc.h>

#include <qpushbutton.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qlineedit.h>
#include <qstring.h>

#include "passwin.h"

int converse(int n, const struct pam_message **msg, 
		struct pam_response **resp, void *data);


PassWin::PassWin(QWidget *parent, const char *name, const QStringList & userList)
 : pwin(parent, name)
{
	conv.conv = converse;
	conv.appdata_ptr = (void *)this;
	cbUserList->insertStringList(userList);
}


PassWin::~PassWin()
{
}

QString PassWin::getUser()
{
	return cbUserList->currentText();
}

QString PassWin::getPass()
{
	return lePasswd->text();
}

bool PassWin::valid_auth()
{
    pam_handle_t *pamh=NULL;
    int retval;

    retval = pam_start("kpctrl_root", getUser(), &conv, &pamh);
        
    if (retval == PAM_SUCCESS)
        retval = pam_authenticate(pamh, 0);

    if (pam_end(pamh,retval) != PAM_SUCCESS) {
        pamh = NULL;
        fprintf(stderr, "kpctrl: failed to release authenticator\n");
        exit(1);
    }

    if (retval != PAM_SUCCESS) {
	    qWarning("PAM authentication failed: %s", 
			    pam_strerror(pamh, retval));
	    return false;
    }

    return true;
}



/* module to auth Parent and Admin */
int converse(int n, const struct pam_message **msg, 
		struct pam_response **resp, void *data)
{
	class PassWin *pwin = (class PassWin *)data;
	struct pam_response *aresp;
	//char buf[PAM_MAX_RESP_SIZE];
	int i;

    	if (n <= 0 || n > PAM_MAX_NUM_MSG)
        	return (PAM_CONV_ERR);

    	if ((aresp = (struct pam_response *)calloc(n, sizeof *aresp)) == NULL)
        	return (PAM_BUF_ERR);

    	for (i = 0; i < n; ++i) {
        	aresp[i].resp_retcode = 0;
        	aresp[i].resp = NULL;
        	switch (msg[i]->msg_style) {
			case PAM_PROMPT_ECHO_OFF:
			aresp[i].resp = strdup((const char *)pwin->getPass());
			if (aresp[i].resp == NULL) {
				qWarning("PAM: cant duplicate password");
				goto fail;
			}
			break;

			case PAM_PROMPT_ECHO_ON:
			qWarning("PAM: prompt echo on not implemented yet");
			break;

			case PAM_ERROR_MSG:
			qWarning("PAM: error msg not implemented yet");
			break;

			case PAM_TEXT_INFO:
			qWarning("PAM: text msg not implemented yet");
			break;

			default:
			goto fail;
		}
	} 

	*resp = aresp;
	return (PAM_SUCCESS);

fail:
	for (i = 0; i < n; ++i) {
		if (aresp[i].resp != NULL) {
			memset(aresp[i].resp, 0, strlen(aresp[i].resp));
			free(aresp[i].resp);
		}
	}
	memset(aresp, 0, n * sizeof *aresp);
	*resp = NULL;
	return (PAM_CONV_ERR);
}

