
#include <qdatetime.h>
#include "rtime.h"
#include "rule.h"

Rule::Rule(const char *timestr)
{
	this->start = NULL;
	this->finish = NULL;
	setString(timestr);
}

Rule::Rule(const char *stimestr, const char *ftimestr)
{
	this->start = NULL;
	this->finish = NULL;
	
	setStartString(stimestr);
	setFinishString(ftimestr);
}


Rule::~Rule()
{
	delete this->start;
	delete this->finish;
}


bool Rule::operator==(class Rule &b) {
	const RTime *bTime;

	bTime = b.getStartTime();
	if (*bTime != *this->start) {
		return false;
	}

	bTime = b.getFinishTime();
	if (*bTime != *this->finish) {
		return false;
	}

	return true;
}


bool Rule::setStart(int hour, int min)
{
	if (this->start)
		delete this->start;

	this->start = new RTime(hour, min);

	return true;
}


bool Rule::setFinish(int hour, int min)
{
	if (this->finish)
		delete this->finish;

	this->finish = new RTime(hour, min);
	return true;
}


RTime *Rule::getStartTime()
{
	return this->start;
}


RTime *Rule::getFinishTime()
{
	return this->finish;
}


const char *Rule::getString()
{
	snprintf(this->timestr, sizeof(this->timestr), "%02d:%02d-%02d:%02d", 
			start->hour(), start->minute(),
			finish->hour(), finish->minute());
	return this->timestr;
}


bool Rule::setStartString(const char *timestr)
{
	bool ret;
	int h;
	int m;
	char *s;
	char *p;

	ret = false;
	s = strdup(timestr);

	// get hour
	if ((p = strstr(s, ":")) == NULL)
		goto out;
	
	*p = '\0';
	h = atoi(s);

	// get minute
	m = atoi(++p);

	ret = setStart(h, m);

out:
	delete s;
	return ret;
}


bool Rule::setFinishString(const char *timestr)
{
	bool ret;
	int h;
	int m;
	char *s;
	char *p;

	ret = false;
	s = strdup(timestr);

	// get hour
	if ((p = strstr(s, ":")) == NULL)
		goto out;
	
	*p = '\0';
	h = atoi(s);

	// get minute
	m = atoi(++p);

	ret = setFinish(h, m);

out:
	delete s;
	return ret;
}


bool Rule::setString(const char *timestr)
{
	/* timestr: 14:40-15:50 */ 
	bool ret;
	char *s, *p;

	ret = false;
	s = strdup(timestr);

	/* get finish hour */
	if ((p = strstr(s, "-")) == NULL)
		goto out;

	*p = '\0';

	ret = setStartString(s);
	if (!ret)
		goto out;

	ret = setFinishString(++p);

out:
	delete s;
	return ret;
}
