
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <iostream>
#include <fstream> 
#include <cstdlib>

#include <qprocess.h>
#include "tiny.h"

using namespace std;

TinyProxy::TinyProxy()
{
	this->config = strdup(TINYPROXYCONF);

	this->proxyEnabled = false;

	this->proxy = NULL;

	this->port = -1;
}


TinyProxy::TinyProxy(char *config)
{
	this->config = strdup(config);
}


TinyProxy::~TinyProxy()
{
	if (this->proxy)
		delete(this->proxy);
}


int TinyProxy::setProxy(const char *server)
{
	int ret;
 
	ret = -1;
	if (server == NULL)
		goto out;

	ret = 0;
	if (this->proxy == NULL) {
		this->proxy = strdup(server);
		goto out;
	}

	delete(this->proxy);
	this->proxy = strdup(server);

out:
	return ret;
		
}


const char *TinyProxy::getProxy()
{
	return this->proxy;
}


int TinyProxy::setPort(int portnum)
{
	this->port = portnum;
	return 0;
}


int TinyProxy::getPort()
{
	return this->port;
}


int TinyProxy::setProxyEnabled()
{
	this->proxyEnabled = true;
	writeConfig();
	restartService();

	return 0;
}
	

bool TinyProxy::isProxyEnabled()
{
	return this->proxyEnabled;
}


int TinyProxy::setProxyDisabled()
{
	this->proxyEnabled = false;
	writeConfig();
	restartService();
	return 0;
}


int TinyProxy::parseUpstream(const char *line)
{
	char *buffer;
	char *pb, *pe;

	buffer = strdup(line);

	this->proxyEnabled = true;
	if (buffer[0] == '#') 
		this->proxyEnabled = false;

	/* find proxy name begin */
	pb = buffer;
	while (*pb != ' ' && *pb != '\0')
		pb++;


	/* skip spaces */
	while(*pb == ' ' && *pb != '\0')
		pb++;


	/* find out where port begin */
	pe = pb;
	while (*pe != ':' && *pe != '\0')
		pe++;

	*pe = '\0';

	this->proxy = strdup(pb);


	/* port begin */
	pb = ++pe; 
	this->port = atoi(pb);

	delete buffer;

	return 0;
}

int TinyProxy::readConfig()
{
	bool found;
	char line[100];

	if (!this->config)
		return -1;

	ifstream infile(this->config);
	if (!infile)
		return -2;

	found = false;
	infile.seekg (0, ios::beg);
	while(infile.getline(line, 100)) {
		/* locate pcontrol signature */
		if (!found) {
			if (strcasecmp(line, SIGNATURE) == 0) {
				found = true;
				continue;
			}
			continue;
		}
		else 
			parseUpstream(line);
	}

	infile.close();
	return 0;
}

int TinyProxy::writeConfig()
{
	const char tmpfile[] = "/tmp/.pcontrol.tmp";
	char line[100];
	char *buffer;
	int size;
	bool found;

	if (!this->config)
		return -1;

	ifstream infile(this->config);
	/* FIXME: use mkstemp */
	ofstream outfile(tmpfile);

	if (!infile || !outfile)
		return -2;

	size = sizeof(line) + 1;
	buffer = new char[size];
	if (!buffer)
		return -3;

	found = false;
	while(infile.getline(line, 100)) {
		/* locate pcontrol signature */
		if (!found) {
			if (strcasecmp(line, SIGNATURE) == 0) {
				found = true;
				snprintf(buffer, size, "%s\n", line); 
				outfile.write(buffer, strlen(buffer));
				continue;
			}
			snprintf(buffer, size, "%s\n", line); 
			outfile.write(buffer, strlen(buffer));
			continue;
		}
		else {
			if (this->proxyEnabled)
				snprintf(buffer, size, "upstream %s:%d", 
						this->proxy, this->port);
			else {
				if (this->proxy) 
					snprintf(buffer, size, "#upstream %s:%d",
							this->proxy, 
							this->port);
				else 
					snprintf(buffer, size, "#upstream");
			}

			outfile.write(buffer, strlen(buffer));
			// final empty line
			snprintf(buffer, size, "\n"); 
			outfile.write(buffer, strlen(buffer));
			break;
		}
	}

	outfile.close();
	infile.close();
	delete buffer;

	infile.open(tmpfile);
	outfile.open(this->config, ios::trunc);

	infile.seekg(0, ifstream::end);
	size = infile.tellg();
	infile.seekg(0);

	// allocate memory for file content
	buffer = new char [size];

	// read content of infile
	infile.read(buffer, size);

	// write to outfile
	outfile.write(buffer, size);
  
	// release dynamically-allocated memory
	delete buffer;

	outfile.close();
	infile.close();

	unlink(tmpfile);

	return 0;
}

bool TinyProxy::restartService()
{
	qWarning("tiny restart\n");
	QProcess tinydaemon(QString("/etc/init.d/tinyproxy"));
	tinydaemon.addArgument(QString("restart"));
	tinydaemon.closeStdin();
	if (!tinydaemon.start()) {
		qWarning("Can't restart tinyproxy");
		return false;
	}

	sleep(1);

	qWarning("dans restart\n");
	QProcess dans(QString("/etc/init.d/dansguardian"));
	dans.addArgument(QString("restart"));
	if (!dans.start()) {
		qWarning("Can't restart dansguardian");
		return -1;
	}

	return true;
}


